require_relative 'fundable'
require_relative 'pledgeable'

module CrowdFund
  class Project
    include Fundable, Pledgeable
  
    attr_reader :funding_goal
    attr_accessor :name, :funding, :backers, :pledges_received
  
    def initialize(name, funding_goal, funding = 0, backers = 0)
      @name = name.capitalize
      @funding_goal = funding_goal
      @funding = funding
      @backers = backers
      @pledges_received = Hash.new(0)
    end

    def self.from_csv(csv_line)
      Project.new(csv_line[0], Integer(csv_line[1]), Integer(csv_line[2]), Integer(csv_line[3]))
    end

    def funding_gap
      funding_goal - funding - pledges
    end

    def name_and_funding
      "#{@name.ljust(20, '.')} £#{@funding + pledges} of £#{@funding_goal}"
    end

    def <=>(project)
      project.funding_gap <=> self.funding_gap
    end

    def to_s
      "Project #{@name} has £#{@funding + pledges} in funding and pledges from #{@backers} backer(s) towards a goal of £#{@funding_goal} leaving a gap of £#{funding_gap} as of #{Time.now.strftime("%F %T")}"
    end
  end


  if __FILE__ == $0
    project1 = Project.new("testbed", 1000, 500, 10)
    puts project1
  end
end